#include "mariamole_auto_generated.h"
#include <Wire.h>
#include <Adafruit_MCP4725.h>


byte PWM_PIN = 3;
int pwm_value;
int voltage;
int pwmlower = 742;
int pwmupper = 2236;
 
Adafruit_MCP4725 dac; 
#define DAC_RESOLUTION    (8)
double kalman_q= 0.05;    // 0.05 do not tamper with this value unless you are looking for adventure ;-)
double kalman_r= 100;	  // default:150   this gets over-ridden by epprom retrieved value
						  // this will only be used if PIN_PPM is NOT defined
						  //  50 = fast but lot of errors (good for sensor testing...but quite a lot of noise)
						  // 200 = medium speed and less false indications (nice medium setting with 
						  //       a silence window (indoor)of -0.2 to +0.2)
						  // 300 = conservative setting ;-)
						  // 500 = still useable maybe for a slow flyer?
						  // 1000 and q=0.001 is it moving at all?? 

void setup() {
  pinMode(PWM_PIN, INPUT);
  Serial.begin(9600);

  // For Adafruit MCP4725A1 the address is 0x62 (default) or 0x63 (ADDR pin tied to VCC)
  // For MCP4725A0 the address is 0x60 or 0x61
  // For MCP4725A2 the address is 0x64 or 0x65
  dac.begin(0x60);  
	
  // Set A2 and A3 as Outputs to make them our GND and Vcc,
  //which will power the MCP4725
  pinMode(A0, INPUT);	
  pinMode(A2, OUTPUT);
  pinMode(A3, OUTPUT);

  digitalWrite(A2, LOW);//Set A2 as GND
  digitalWrite(A3, HIGH);//Set A3 as Vcc	
  
}
 
float kalman_update(int measurement)
{
  //static int lcnt=0;
  static float x=measurement; //value
  static float p=100; //estimation error covariance
  static float k=0; //kalman gain

  // update the prediction value
  p = p + kalman_q;

  // update based on measurement
  k = p / (p + kalman_r);
  x = x + k * (measurement - x);
  p = (1 - k) * p;

  return x;
}

void loop() {
  pwm_value = pulseIn(PWM_PIN, HIGH);
  
  voltage = kalman_update(map(pwm_value, pwmlower, pwmupper, 0, 4095));
	
 
  dac.setVoltage(voltage, false);
	
  //int sensorValue = analogRead(A0);
  //Serial.println(sensorValue);
  //Serial.println(voltage);
	
  int sensorValue = analogRead(A0);
  // Convert the analog reading (which goes from 0 - 1023) to a voltage (0 - 5V):
  float rvoltage = sensorValue * (5.0 / 1023.0);
  // print out the value you read:
  Serial.println(rvoltage);	
	
}



